/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/




// This handles the actual drawing of objects

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"


// The editing dialogs
extern CDlgERCListBox theERCListBox;

////////// The default actions for each object ///////////


CDrawingObject::CDrawingObject( CTinyCadDoc *pDesign )
{
	m_pDesign = pDesign;
	a = CPoint(0,0);
	b = CPoint(0,0);
	Delete_Level = -1;
	Undo_Level = -1;
	next = NULL;
	segment = 0;
}

BOOL CDrawingObject::IsConstruction()
{
	return FALSE;
}


void CDrawingObject::EndPlace(CPoint)
{

}


int  CDrawingObject::CanUseOptions()
{
  return 0;
}

// Is a point pointing at one of our fields?
int CDrawingObject::IsInsideField(CPoint)
{
	return -1;
}

void CDrawingObject::MoveField(int w, CPoint r)
{
	// Do nothing
}


// Set the new layer when the options change
void CDrawingObject::NewOptions()
{
}

void CDrawingObject::TagResources()
{

}



// Read the CPoints from a file (for Win16 compatibility)
CPoint CDrawingObject::ReadPoint(CArchive &theFile)
{   
	CPoint r;

	theFile >> r;
	
	return r;
}


void CDrawingObject::WritePoint(CArchive &theFile, CPoint a)
{
	theFile << a;
}



// Place the object into the design
void CDrawingObject::Place(CPoint p, CPoint)
{
  if (segment) {
	a=p;
	b=p;
	segment=!segment;
	Display();
  } else {
	b=p;
	Display();
	Store();
	a=b;
  }
}

// Find a string in this object
CString CDrawingObject::Find(const char *)
{
	return "";
}


// Move the object about the design
void CDrawingObject::Move(CPoint p)
{
  if (!segment) {
	Display();
	b=p;
	Display();
  }
}


// Cancel editing this part of the object in the design
BOOL CDrawingObject::End()
{
  // If necessary remove the line from the screen
  Display();

  // Signal this is the end of this object
  return FALSE;
}

void CDrawingObject::AfterEnd( CPoint, CPoint )
{

}


void CDrawingObject::BeginEdit()
{
}

void CDrawingObject::EndEdit()
{
  Display();
}

BOOL CDrawingObject::CanEdit()
{
  return FALSE;
}

BOOL CDrawingObject::DoesExist()
{
  return Undo_Level<=m_pDesign->GetUndoLevel() && (Delete_Level==-1 || Delete_Level>m_pDesign->GetUndoLevel() );
}

ObjType CDrawingObject::GetType()
{
  return xAnotation;
}


CString CDrawingObject::GetName()
{
  return "??";
}

void CDrawingObject::Display( BOOL erase )
{
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, 0 );
}

void CDrawingObject::Paint( Context &dc, paint_options options)
{
}

void CDrawingObject::Store()
{
  m_pDesign->SetRepeat(NULL);
}

int CDrawingObject::IsInside(int left,int right,int top,int bottom)
{
  return !( (a.x<left && b.x<=left) || (a.x>right && b.x>=right)
        || (a.y<top && b.y<=top) || (a.y>bottom && b.y>=bottom));
}


// Save the object to a file
void CDrawingObject::Save(CArchive& archive )
{
  WritePoint(archive,a);
  WritePoint(archive,b);
}


// Load the object to a file
void CDrawingObject::Load(CArchive& archive )
{
 	a = ReadPoint(archive);
   	b = ReadPoint(archive);
}




// Rotate this object about a point
void CDrawingObject::Rotate(CPoint p,int dir)
{
  // Translate this point so the rotational point is the origin
  a = CPoint(a.x-p.x,a.y-p.y);
  b = CPoint(b.x-p.x,b.y-p.y);

  // Perfrom the rotation
  switch (dir) {
	case 2: // Left
		a = CPoint(a.y,-a.x);
		b = CPoint(b.y,-b.x);
		break;		
	case 3: // Right
		a = CPoint(-a.y,a.x);
		b = CPoint(-b.y,b.x);
		break;
	case 4: // Mirror
		a = CPoint(-a.x,a.y);
		b = CPoint(-b.x,b.y);
		break;
  }

  // Re-translate the points back to the original location
  a = CPoint(a.x+p.x,a.y+p.y);
  b = CPoint(b.x+p.x,b.y+p.y);
  
}

int CDrawingObject::SetCursor( CPoint p )
{
	return -1;
}

int CDrawingObject::SetCursorEdit( CPoint p )
{
	BOOL r = IsInside(p.x,p.x,p.y,p.y);
	return r ? 8 : -1;
}



/////// The CDrawRectOutline class /////////




// Redraw this object (including the handles)
void CDrawRectOutline::Display( BOOL erase  )
{
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, 8 );
}

void CDrawRectOutline::DrawHandles( Context &dc )
{
	// Put some handles around this object
	CRect r(a.x,a.y,b.x,b.y);

	CRectTracker	tracker( dc.theTransform.Scale(r), CRectTracker::dottedLine | CRectTracker::resizeOutside  );
	tracker.Draw( dc.dc );
}


// Move fields of this object about
int CDrawRectOutline::IsInsideField(CPoint p)
{
	CRectTracker	tracker( CRect(a.x,a.y,b.x,b.y), CRectTracker::dottedLine | CRectTracker::resizeOutside  );
	int r = tracker.HitTest( p );

	if (r == CRectTracker::hitMiddle)
	{
		r = CRectTracker::hitNothing;
	}

	return r;
}

void CDrawRectOutline::MoveField(int w, CPoint r)
{
	Display();

	CRect rect( a.x, a.y, b.x, b.y );

	switch (w)
	{
	case CRectTracker::hitTopLeft:
		rect.left += r.x;
		rect.top += r.y;
		break;
	case CRectTracker::hitTopRight:
		rect.right += r.x;
		rect.top += r.y;
		break;
	case CRectTracker::hitBottomRight:
		rect.right += r.x;
		rect.bottom += r.y;
		break;
	case CRectTracker::hitBottomLeft:
		rect.left += r.x;
		rect.bottom += r.y;
		break;	
	case CRectTracker::hitTop:
		rect.top += r.y;
		break;		
	case CRectTracker::hitRight:
		rect.right += r.x;
		break;		
	case CRectTracker::hitBottom:
		rect.bottom += r.y;
		break;		
	case CRectTracker::hitLeft:
		rect.left += r.x;
		break;		
	}

	a.x = rect.left;
	a.y = rect.top;
	b.x = rect.right;
	b.y = rect.bottom;

	Display();
}

int CDrawRectOutline::SetCursorEdit( CPoint p )
{
	CRect r(a.x,a.y,b.x,b.y);
	r.NormalizeRect();
	CRectTracker	tracker( r, CRectTracker::dottedLine | CRectTracker::resizeOutside  );
	return tracker.HitTest( p );
}


/////// The Object classes /////////



// The Rectangle Class //

// The Constructor
CDrawSquare::CDrawSquare( CTinyCadDoc *pDesign )
: CDrawRectOutline( pDesign )
{
	segment=1;
	Undo_Level=0;
	Delete_Level=-1;
	Style = m_pDesign->GetOptions()->GetCurrentStyle();
}



ObjType CDrawSquare::GetType()
{
  return xSquareEx2;
}

CString CDrawSquare::GetName()
{
  return "Rectangle";
}


void CDrawSquare::TagResources()
{
   m_pDesign->GetOptions()->TagStyle(Style);
}


BOOL CDrawSquare::End()
{
  BOOL r = !segment;

  Display();
  segment=1;
  
  return r;
}

// Save the rectangle to a file
void CDrawSquare::Save(CArchive& archive )
{
  WritePoint(archive,a);
  WritePoint(archive,b);

  archive << Style;
}


// Load the rectange from a file
void CDrawSquare::Load(CArchive& archive )
{
 	a = ReadPoint(archive);
   	b = ReadPoint(archive);

  archive >> Style;
  Style = m_pDesign->GetOptions()->GetNewStyleNumber(Style);
}

// The old version of this load
void CDrawSquare::OldLoad(CArchive& archive, int tp )
{
  BYTE LineThickness = 1;
  a = ReadPoint(archive);
  b = ReadPoint(archive);

  if (tp == xSquareEx) {
  	archive >> LineThickness;
  }

  LineStyle l;
  l.Colour = cBLACK;
  l.Style = PS_SOLID;
  l.Thickness = LineThickness;
  Style = m_pDesign->GetOptions()->AddStyle(&l);
}

// Change the setting due to the current options
void CDrawSquare::NewOptions()
{
 	Display();
	Style = m_pDesign->GetOptions()->GetCurrentStyle();
	Display();
}


void CDrawSquare::Paint(Context &dc,paint_options options)
{

  CPoint sma = a;
  CPoint smb = b;

  dc.SelectBrush();
  dc.SelectPen(m_pDesign->GetOptions()->GetStyle(Style),options);
  dc.SetROP2(R2_COPYPEN);
  dc.Rectangle(CRect(sma.x,sma.y,smb.x,smb.y));

  if (options == draw_handles)
  {
	  DrawHandles( dc );

  }

}


// Store the rectangle in the drawing
void CDrawSquare::Store()
{
  CDrawSquare *NewObject;

  NewObject = new CDrawSquare(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);
  
  segment=1;

  m_pDesign->SetRepeat(NULL);
}



// The Circle Class //

// The Constructor
CDrawCircle::CDrawCircle(CTinyCadDoc *pDesign)
: CDrawRectOutline( pDesign )
{
	segment=1;
	Undo_Level=0;
	Delete_Level=-1;
	Style = m_pDesign->GetOptions()->GetCurrentStyle();
}

ObjType CDrawCircle::GetType()
{
  return xCircleEx2;
}

CString CDrawCircle::GetName()
{
  return "Ellipse";
}

void CDrawCircle::TagResources()
{
   m_pDesign->GetOptions()->TagStyle(Style);
}


// Save the rectangle to a file
void CDrawCircle::Save(CArchive& archive )
{
  WritePoint(archive,a);
  WritePoint(archive,b);

  archive << Style;
}


// Load the object to a file
void CDrawCircle::Load(CArchive& archive )
{
 	a = ReadPoint(archive);
   	b = ReadPoint(archive);

  archive >> Style;
  Style = m_pDesign->GetOptions()->GetNewStyleNumber(Style);
}

// The old version of this load
void CDrawCircle::OldLoad(CArchive& archive, int tp )
{
  BYTE LineThickness = 1;
  a = ReadPoint(archive);
  b = ReadPoint(archive);

  if (tp == xCircleEx) {
  	archive >> LineThickness;
  }

  LineStyle l;
  l.Colour = cBLACK;
  l.Style = PS_SOLID;
  l.Thickness = LineThickness;
  Style = m_pDesign->GetOptions()->AddStyle(&l);

}

// Change the setting due to the current options
void CDrawCircle::NewOptions()
{
 	Display();
	Style = m_pDesign->GetOptions()->GetCurrentStyle();
	Display();
}




BOOL CDrawCircle::End()
{
  BOOL r = !segment; 

  Display();
  segment=1;
  
  return r;
}


// The rectangle area slightly re-written
#define theArea CRect(a.x,a.y,b.x,b.y)

void CDrawCircle::Paint(Context &dc,paint_options options)
{
  dc.SelectBrush();
  dc.SelectPen(m_pDesign->GetOptions()->GetStyle(Style),options);
  dc.SetROP2(R2_COPYPEN);

  dc.Ellipse(theArea);

  if (options == draw_handles)
  {
	  DrawHandles( dc );

  }
}


// Store the ellipse in the drawing
void CDrawCircle::Store()
{
  CDrawCircle *NewObject;

  NewObject = new CDrawCircle(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);

  segment=1;

  m_pDesign->SetRepeat(NULL);

}



////// The Arc Class //////


// The Constructor
CDrawArcSegment::CDrawArcSegment(CTinyCadDoc *pDesign)
: CDrawRectOutline( pDesign )
{
	segment=1;
	Undo_Level=0;
	Delete_Level=-1;
	Style = m_pDesign->GetOptions()->GetCurrentStyle();
}

ObjType CDrawArcSegment::GetType()
{
  return xArcEx2;
}

CString CDrawArcSegment::GetName()
{
  return "Arc";
}

void CDrawArcSegment::TagResources()
{
   m_pDesign->GetOptions()->TagStyle(Style);
}


// Save the rectangle to a file
void CDrawArcSegment::Save(CArchive& archive )
{
  WritePoint(archive,a);
  WritePoint(archive,b);

  archive << Style;
}


// Load the object to a file
void CDrawArcSegment::Load(CArchive& archive )
{
 	a = ReadPoint(archive);
   	b = ReadPoint(archive);

  archive >> Style;
  Style = m_pDesign->GetOptions()->GetNewStyleNumber(Style);
}

// The old version of this load
void CDrawArcSegment::OldLoad(CArchive& archive, int tp )
{
  BYTE LineThickness = 1;
  a = ReadPoint(archive);
  b = ReadPoint(archive);

  if (tp == xArcEx) {
  	archive >> LineThickness;
  }

  LineStyle l;
  l.Colour = cBLACK;
  l.Style = PS_SOLID;
  l.Thickness = LineThickness;
  Style = m_pDesign->GetOptions()->AddStyle(&l);
}

// Change the setting due to the current options
void CDrawArcSegment::NewOptions()
{
 	Display();
	Style = m_pDesign->GetOptions()->GetCurrentStyle();
	Display();
}



BOOL CDrawArcSegment::End()
{
  BOOL r = !segment;

  Display();
  segment=1;
  
  return r;
}

void CDrawArcSegment::Paint(Context &dc,paint_options options)
{
  dc.SelectBrush();
  dc.SelectPen(m_pDesign->GetOptions()->GetStyle(Style),options);
	dc.SetROP2(R2_COPYPEN);

  dc.QuaterArc(a,b);

  if (options == draw_handles)
  {
	  DrawHandles( dc );

  }

}


// Store the arc in the drawing
void CDrawArcSegment::Store()
{
  CDrawArcSegment *NewObject;

  NewObject = new CDrawArcSegment(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);

  segment=1;

  m_pDesign->SetRepeat(NULL);
}


// The Error Class //

#define ERRORSIZE	6

CDrawError::CDrawError(CTinyCadDoc *pDesign, CPoint NewA,int q)
: CDrawingObject( pDesign )
{
	a = NewA;
	b = NewA;
	segment=0;
	ErrorNumber = q;
	Undo_Level=0;
	Delete_Level=-1;
}


ObjType CDrawError::GetType()
{
  return xError;
}

CString CDrawError::GetName()
{
  return "Error";
}

int CDrawError::IsInside(int left,int right,int top,int bottom)
{
  return !( (a.x-ERRORSIZE<left && a.x+ERRORSIZE<=left) || (a.x-ERRORSIZE>right && a.x+ERRORSIZE>=right)
        || (a.y-ERRORSIZE<top && a.y+ERRORSIZE<=top) || (a.y-ERRORSIZE>bottom && a.y+ERRORSIZE>=bottom));
}

void CDrawError::Paint(Context &dc,paint_options options)
{
  dc.SelectBrush();

  switch (options)
  {
  case draw_selected:
  case draw_handles:
	  dc.SelectPen(PS_SOLID,5,cSELECT);
	  break;
  case draw_selectable:
	  dc.SelectPen(PS_SOLID,5,cBUS_CLK);
	  break;
  default:
	  dc.SelectPen(PS_SOLID,5,cBUS);
  }

  dc.SetROP2(R2_COPYPEN);

  dc.Ellipse(CRect(a.x+ERRORSIZE,a.y+ERRORSIZE,a.x-ERRORSIZE,a.y-ERRORSIZE));
}


void CDrawError::BeginEdit()
{
	theERCListBox.SetSelect(ErrorNumber);
}

void CDrawError::EndEdit()
{
}

BOOL CDrawError::CanEdit()
{
  return TRUE;
}



// Store the error in the design
void CDrawError::Store()
{
  CDrawError *NewObject;

  NewObject = new CDrawError(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);
  
  segment=1;

  m_pDesign->SetRepeat(NULL);
}



////// The tag object type //////

// Save the object to a file
void CDrawTag::Save(CArchive& archive )
{
  WritePoint(archive,a);
  archive << TagName;
}


// Load the object to a file
void CDrawTag::Load(CArchive& archive )
{
  	a = b = ReadPoint(archive);
  	archive >> TagName;
}






