/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "option.h"
#include "revision.h"
#include "registry.h"
#include "TinyCadView.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "diag.h"



CDlgERCListBox theERCListBox;




////// The GetFindBox dialog //////

BEGIN_MESSAGE_MAP( CDlgGetFindBox, CDialog )
	ON_LBN_SELCHANGE(FINDGET_LIST , OnListSelect )
END_MESSAGE_MAP()

BOOL CDlgGetFindBox::OnInitDialog()
{
	CLibraryFile *aLibrary = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();
	// Build the list box
	CListBox *theListBox = (CListBox *)GetDlgItem(FINDGET_LIST);
	theListBox->ResetContent();

	// Ensure the string is in Lower case for searching
	SearchString.MakeLower();

	while (aLibrary!=NULL) 
	{
		aLibrary->Find(SearchString,theListBox);
		aLibrary = aLibrary->next;
	}

	return TRUE;
}


// End the dialog when an item is clicked on
void CDlgGetFindBox::OnListSelect()
{
	CListBox *theListBox = (CListBox *)GetDlgItem(FINDGET_LIST);
	int WhichItem;

 	if ((WhichItem = theListBox->GetCurSel())!=LB_ERR) 
	{
		theListBox->GetText(WhichItem,SelectString);
		EndDialog(IDOK);
	}
}



////// The GetBox dialog //////

BEGIN_MESSAGE_MAP( CDlgGetBox, CDialog )
	ON_COMMAND(GETBOX_FIND, OnFind)
END_MESSAGE_MAP()


void CDlgGetBox::OnOK()
{
  GetDlgItemText( GETBOX_NAME, String.GetBuffer(64), 64 );
  String.ReleaseBuffer();

  EndDialog( IDOK );
}

void CDlgGetBox::OnFind()
{
	GetDlgItemText( GETBOX_NAME, String.GetBuffer(64), 64 );
	String.ReleaseBuffer();

	CDlgGetFindBox theDialog(String,this);

	if (theDialog.DoModal() == IDOK) 
	{
		CString SelectString = theDialog.GetSelectString();
		SelectString = SelectString.Left(SelectString.Find(' '));
		SetDlgItemText( GETBOX_NAME, SelectString );
	}
}


////// The TagBox dialog //////

void CDlgFindBox::OnOK()
{
  GetDlgItemText( FINDBOX_STRING, String.GetBuffer(64), 64 );
  String.ReleaseBuffer();

  EndDialog( IDOK );
}


////// The TagBox dialog //////

void CDlgTagBox::OnOK()
{
  GetDlgItemText( TAGBOX_TAGNAME, String.GetBuffer(64), 64 );
  String.ReleaseBuffer();

  EndDialog( IDOK );
}



////// The JumpTagBox dialog //////

BEGIN_MESSAGE_MAP( CDlgJumpTagBox, CDialog )
	ON_BN_CLICKED( JUMPTAGBOX_DELETE, OnDelete )
END_MESSAGE_MAP()


BOOL CDlgJumpTagBox::OnInitDialog()
{
	// Build the list box
	CListBox *theListBox = (CListBox *)GetDlgItem(TAGJUMP_LIST);
	theListBox->ResetContent();
 
 	CDrawingObject *pointer = aDesign->GetHead();
	while (pointer!=NULL) {
		if (pointer->GetType() == xTag) {
			#define theTag ((CDrawTag *)pointer)
			theListBox->AddString(theTag->GetTagName());
		}
		pointer = pointer->next;
	}
	 

	return TRUE;
}


void CDlgJumpTagBox::OnDelete()
{
    int WhichItem;
	CListBox *theListBox = (CListBox *)GetDlgItem(TAGJUMP_LIST);

  	if ((WhichItem = theListBox->GetCurSel())!=LB_ERR) {
		theListBox->GetText(WhichItem,String);

		CDrawingObject *pointer=aDesign->GetHead();
		while (pointer!=NULL && (pointer->GetType()!=xTag || ((CDrawTag *)pointer)->GetTagName()!=String) )
			pointer=pointer->next;

		if (pointer!=NULL) {
			aDesign->Select(pointer);
			aDesign->SelectDelete();
		}

		theListBox->DeleteString(WhichItem);
	}
}

void CDlgJumpTagBox::OnOK()
{
    int WhichItem;
	CListBox *theListBox = (CListBox *)GetDlgItem(TAGJUMP_LIST);

  	if ((WhichItem = theListBox->GetCurSel())!=LB_ERR) {

		theListBox->GetText(WhichItem,String);
		EndDialog(IDOK);                      

	} else
		EndDialog(IDCANCEL);
}






////// The Page Size dialog Box //////
BEGIN_MESSAGE_MAP( CDlgPageSizeBox, CDialog )
	ON_BN_CLICKED(PAGESIZE_A4, OnChange )
	ON_BN_CLICKED(PAGESIZE_A3, OnChange )
	ON_BN_CLICKED(PAGESIZE_A2, OnChange )
	ON_BN_CLICKED(PAGESIZE_A1, OnChange )
	ON_BN_CLICKED(PAGESIZE_PRINTER, OnChange )
	ON_BN_CLICKED(PAGESIZE_PORT, OnMakePort )
	ON_BN_CLICKED(PAGESIZE_LAND, OnMakeLand )
	ON_BN_CLICKED(PAGESIZE_PRINTERSET, OnPrinterSet )
	ON_EN_CHANGE(PAGESIZE_WIDTH, OnChangeText )
	ON_EN_CHANGE(PAGESIZE_HEIGHT, OnChangeText )
END_MESSAGE_MAP()


// Arrays storing the sizes of the pre-defined pages
int Page_SizesA[] = { 297, 420, 594, 840,   297 };
int Page_SizesB[] = { 210, 297, 420, 594,   210 };
const int NumberOfSizes = 5;


BOOL CDlgPageSizeBox::SetPrinter(BOOL display_dialogue)
{
	HDC hdc = NULL;

  // Get the dimensions of the default printer's page
	CPrintDialog pdlg( FALSE, 0, AfxGetMainWnd() );

	// Create a printer dialogue
	if (display_dialogue)
	{
		if (pdlg.DoModal())
		{
			hdc = pdlg.GetPrinterDC();
		}
	}
	else
	{
		if (AfxGetApp()->GetPrinterDeviceDefaults(&pdlg.m_pd))
		{
			if (pdlg.m_pd.hDC == NULL)
			{
				// call CreatePrinterDC if DC was not created by above
				hdc = pdlg.CreatePrinterDC();
			}
		}
	}

	if (hdc) 
	{
		// Change the scaling to print correct size
		int x=::GetDeviceCaps(hdc,HORZSIZE);
		int y=::GetDeviceCaps(hdc,VERTSIZE);
		Page_SizesA[NumberOfSizes-1]=max(x,y);
		Page_SizesB[NumberOfSizes-1]=min(x,y);
	}

	::DeleteDC( hdc );

	return hdc != NULL; 
}


BOOL CDlgPageSizeBox::OnInitDialog()
{
  PageSizeChanged = FALSE;
  PageSetupChanged = FALSE;

  SetPrinter( FALSE );

  // Set the correct settings in the dialog box
  SetDlgItemInt(PAGESIZE_WIDTH,Size.x/PIXELSPERMM);
  SetDlgItemInt(PAGESIZE_HEIGHT,Size.y/PIXELSPERMM);

  return TRUE;
}


void CDlgPageSizeBox::OnPrinterSet()
{

  if (SetPrinter( TRUE ))
  {
	  // Update the dialog to reflect any changes
	  CheckRadioButton(PAGESIZE_A4,PAGESIZE_PRINTER,PAGESIZE_PRINTER);
	  OnChange();

	  PageSetupChanged = TRUE;
  }
}


void CDlgPageSizeBox::OnMakePort()
{
  char BufferWidth[SIZESTRING],BufferHeight[SIZESTRING];

  // If it is not already portrait then swap width and height
  if (GetDlgItemInt(PAGESIZE_WIDTH)>GetDlgItemInt(PAGESIZE_HEIGHT)) {
	GetDlgItemText(PAGESIZE_WIDTH, BufferWidth, sizeof(BufferWidth) );
	GetDlgItemText(PAGESIZE_HEIGHT, BufferHeight, sizeof(BufferHeight) );
	SetDlgItemText(PAGESIZE_HEIGHT, BufferWidth);
	SetDlgItemText(PAGESIZE_WIDTH, BufferHeight);
  }

  PageSizeChanged = TRUE;
}

void CDlgPageSizeBox::OnMakeLand()
{
  char BufferWidth[SIZESTRING],BufferHeight[SIZESTRING];

  // If it is not already landscape then swap width and height
  if (GetDlgItemInt(PAGESIZE_WIDTH)<GetDlgItemInt(PAGESIZE_HEIGHT)) {
	GetDlgItemText(PAGESIZE_WIDTH, BufferWidth, sizeof(BufferWidth) );
	GetDlgItemText(PAGESIZE_HEIGHT, BufferHeight, sizeof(BufferHeight) );
	SetDlgItemText(PAGESIZE_HEIGHT, BufferWidth);
	SetDlgItemText(PAGESIZE_WIDTH, BufferHeight);
  }

  PageSizeChanged = TRUE;
}


void CDlgPageSizeBox::OnChange()
{
  int which = GetCheckedRadioButton(PAGESIZE_A4,PAGESIZE_PRINTER)-PAGESIZE_A4;

  int a = Page_SizesA[which];
  int b = Page_SizesB[which];

  if (GetCheckedRadioButton(PAGESIZE_LAND,PAGESIZE_PORT)==PAGESIZE_PORT) {
	SetDlgItemInt(PAGESIZE_HEIGHT,a);
	SetDlgItemInt(PAGESIZE_WIDTH,b);
  } else {
	SetDlgItemInt(PAGESIZE_WIDTH,a);
	SetDlgItemInt(PAGESIZE_HEIGHT,b);
  }

  PageSizeChanged = TRUE;
}

void CDlgPageSizeBox::OnChangeText()
{
  int NewWidth = GetDlgItemInt(PAGESIZE_WIDTH);
  int NewHeight = GetDlgItemInt(PAGESIZE_HEIGHT);

  if (NewWidth<NewHeight)
	CheckRadioButton(PAGESIZE_LAND,PAGESIZE_PORT,PAGESIZE_PORT);
  else
	CheckRadioButton(PAGESIZE_LAND,PAGESIZE_PORT,PAGESIZE_LAND);

  // Do these values correspond to a predefined value?
  int a = max(NewWidth,NewHeight);
  int b = min(NewWidth,NewHeight);

  for (int lp=0;lp!=NumberOfSizes; lp++)
	if (a == Page_SizesA[lp] && b == Page_SizesB[lp])
		CheckRadioButton(PAGESIZE_A4,PAGESIZE_PRINTER,PAGESIZE_A4+lp);

  PageSizeChanged = TRUE;
}


void CDlgPageSizeBox::OnOK()
{
  Size.x = GetDlgItemInt(PAGESIZE_WIDTH)*PIXELSPERMM;
  Size.y = GetDlgItemInt(PAGESIZE_HEIGHT)*PIXELSPERMM;

  if (PageSetupChanged || PageSizeChanged) {
	Registry theRegistry;

	if (PageSizeChanged)
		theRegistry.SavePageSize(Size);
  }

  if (Size.x < 10 || Size.y < 10)
	EndDialog(IDCANCEL);
  else
	EndDialog(IDOK);
}




////// The ERC Box Dialog //////


BEGIN_MESSAGE_MAP( CDlgERCListBox, CDialog )
	ON_LBN_SELCHANGE(ERCLIST_LIST , OnClick )
	ON_BN_CLICKED(IDOK , Close )
END_MESSAGE_MAP()


CDlgERCListBox::CDlgERCListBox()
{
  open=FALSE;
  stop=FALSE;
}

void CDlgERCListBox::Open(CTinyCadDoc *pDesign)
{
	m_pDesign = pDesign;

  if (!open) {
  	Create(IDD_ERCLIST,AfxGetMainWnd());

	theListBox = (CListBox *)GetDlgItem(ERCLIST_LIST);
	theListBox->ResetContent();
  }
  open = TRUE;
  stop = FALSE;
}

// Close the dialog window 
void CDlgERCListBox::Close()
{
  if (open) {
	DestroyWindow();
	// Now remove all the errors from the design
	m_pDesign->DeleteErrors();
	open = FALSE;
  }
}


void CDlgERCListBox::AddString(CString NewString)
{
  if (open) {
	theListBox = (CListBox *)GetDlgItem(ERCLIST_LIST);
	theListBox->AddString(NewString);
  }
}

void CDlgERCListBox::SetSelect(int NewSelect)
{
  if (open) {
	stop = TRUE;
	theListBox = (CListBox *)GetDlgItem(ERCLIST_LIST);
	theListBox->SetCurSel(NewSelect);
	stop = FALSE;
  }
}

void CDlgERCListBox::OnClick()
{
  // Do we notice this?
  if (stop)
	return;

  // Get the item which has been selected
  int WhichItem;
  theListBox = (CListBox *)GetDlgItem(ERCLIST_LIST);
  if ((WhichItem = theListBox->GetCurSel())==LB_ERR)
	return;
 
  // Now search the design for this error
  CDrawingObject *pointer = m_pDesign->GetHead();
  while (pointer!=NULL) {
	if (pointer->GetType()==xError && ((CDrawError *)pointer)->GetErrorNumber()==WhichItem) {
		m_pDesign->SetScrollCentre(pointer->a);
		break;
	}
	pointer = pointer->next;
  }
}

