/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#ifndef __DIAG_H__
#define __DIAG_H__

#define SIZESTRING 254

class CDrawingObject;
class CDrawMethod;
class CDrawPin;
class CDrawText;
class CDrawPower;












// The dialog for the do you want to save?
class CDlgGetFindBox : public CDialog {
	CString SearchString;
	CString SelectString;

public:	
 	CDlgGetFindBox(const char *ns,CWnd *pParentWnd = NULL) : CDialog( IDD_GETFIND, pParentWnd ) { SearchString = ns; }
	BOOL OnInitDialog();
	CString GetSelectString() { return SelectString; }

	afx_msg void OnListSelect();
	DECLARE_MESSAGE_MAP()
};


// The dialog for getting the name of a symbol
class CDlgGetBox : public CDialog {

	CString	String;

public:
	CDlgGetBox(CWnd *pParentWnd) : CDialog( IDD_GET, pParentWnd ) { }

	char *GetString() { return String.GetBuffer(1); }

	void OnFind();
	virtual void OnOK();

	DECLARE_MESSAGE_MAP()
};


// The dialog for getting the name of a new find string
class CDlgFindBox : public CDialog {

	CString	String;

public:
	CDlgFindBox(CWnd *pParentWnd) : CDialog( IDD_FIND, pParentWnd ) { }

	CString GetString() { return String; }

	virtual void OnOK();
};


// The dialog for getting the name of a new tag
class CDlgTagBox : public CDialog {

	CString	String;

public:
	CDlgTagBox(CWnd *pParentWnd) : CDialog( IDD_TAG, pParentWnd ) { }

	CString GetString() { return String; }

	virtual void OnOK();
};



// The dialog for getting the name of a tag in existance
class CDlgJumpTagBox : public CDialog {

	CTinyCadDoc	*aDesign;
	CString	String;

public:
	CDlgJumpTagBox(CWnd *pParentWnd, CTinyCadDoc *newDesign) : CDialog( IDD_JUMPTAG, pParentWnd ) { aDesign = newDesign; }

	CString GetString() { return String; }

	virtual void OnOK();
	virtual BOOL OnInitDialog();
	afx_msg void OnDelete();

    DECLARE_MESSAGE_MAP()
};






// The dialog for editing the text object
class CDlgPageSizeBox : public CDialog {

	CPoint Size;
	BOOL PageSizeChanged;
	BOOL PageSetupChanged;

	BOOL SetPrinter(BOOL display_dialogue);

public:
	CDlgPageSizeBox(CWnd *pParentWnd, CPoint CurrentSize) : CDialog( IDD_PAGESIZE, pParentWnd )
		{ Size = CurrentSize; }
	CPoint GetSize() { return Size; }

	void OnMakePort();
	void OnMakeLand();
	void OnPrinterSet();
	void OnChange();
	void OnChangeText();

	virtual void OnOK();
	virtual BOOL OnInitDialog();

    DECLARE_MESSAGE_MAP()
};







// The dialog for the error edit
class CDlgERCListBox : public CDialog {
	int open;
	CListBox *theListBox;
	BOOL stop;
	CTinyCadDoc *m_pDesign;
public:	
	CDlgERCListBox();
	void Open(CTinyCadDoc *pDesign);
	void Close();
	void AddString(CString);
	void SetSelect(int);
	void OnClick();

	DECLARE_MESSAGE_MAP()
};





	////////// The rebar edit dialogs //////////



class CEditDlg : public CDialog
{
private:
	CDrawingObject *m_pObject;
protected:
	CTinyCadDoc	*m_pDesign;

	int opens;

	void Show( CTinyCadDoc *pDesign, CDrawingObject *pObject );
	CDrawingObject *getObject();

public:
	CEditDlg( )
	{
		m_pDesign = NULL;
		m_pObject = NULL;
		opens = 0;
	}

	void changeSelected( CDrawingObject *previous, CDrawingObject *pObject );

	void Close();
};

class CEditDlgNoTool : public CEditDlg 
{
public:
	void Create();

};

// The dialog for editing the text object
class CEditDlgBusNameEdit : public CEditDlg {

	BOOL stop;

public:

	CEditDlgBusNameEdit() { opens=0; }
	void Create();
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	void ReFocus();

	// Stop Enter closing this dialog
	void OnOK() { }

	afx_msg void OnChange();
	
    DECLARE_MESSAGE_MAP()
};



// The dialog for editing the power object
class CEditDlgLineEdit : public CEditDlg {

	int angle;
public:
	int mode;

	CEditDlgLineEdit()
	{
	  angle=LINEBOX_90;
	}

	void Create();
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	int GetAngle() { return GetCheckedRadioButton(LINEBOX_FREE,LINEBOX_45); }

	// Stop Enter closing this dialog
	void OnOK() { }
};


// The dialog for editing the method object
class CEditDlgMethodEdit : public CEditDlg {

	afx_msg void OnChangeText();
	afx_msg void OnChangeRotation();
	afx_msg void OnChangePart();
	BOOL stop;
public:

	CEditDlgMethodEdit()  {  opens=0; }
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	void Create();

	int GetDir();

	// Stop Enter closing this dialog
	void OnOK() { }

	DECLARE_MESSAGE_MAP()
};



// The dialog for the pin object
class CEditDlgPinEdit : public CEditDlg {
	BOOL stop;

public:
	CEditDlgPinEdit();
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	void Create();
	void ReFocus();

	// Stop Enter closing this dialog
	void OnOK() { }

	afx_msg void OnChange();
	
    DECLARE_MESSAGE_MAP()
};




// The dialog for editing the power object
class CEditDlgPowerEdit : public CEditDlg {
	BOOL stop;
public:
	CEditDlgPowerEdit(){ opens = 0; }
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	void Create();

	// Stop Enter closing this dialog
	void OnOK() { }

	afx_msg void OnChange();
	
    DECLARE_MESSAGE_MAP()
};



// The dialog for the rotate block
class CEditDlgRotateBox : public CEditDlg {

public:
	CEditDlgRotateBox() { opens = 0; }
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	void Create();

	afx_msg void OnLeft();
	afx_msg void OnRight();
	afx_msg void OnMirror();
	
    DECLARE_MESSAGE_MAP()
};




// The dialog for editing the text object
class CEditDlgTextEdit : public CEditDlg {

	BOOL stop;

public:

	CEditDlgTextEdit() { opens=0; }
	void Open(CTinyCadDoc *pDesign, CDrawingObject *pObject);
	void Create();
	void ReFocus();

	// Stop Enter closing this dialog
	void OnOK() { }

	afx_msg void OnChange();
	
    DECLARE_MESSAGE_MAP()
};




#endif
