/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// This is an interface to the registry database



#ifndef __REGISTRY_H__
#define __REGISTRY_H__

#include "revision.h"
#include "io.h"

// Each registry entry is placed into HKEY_CURRENT_USER\Software\TinyCAD\Name
// (where name is specified on construction of registry)
// They are then sub divided into keys.  Each key is a single string in the database

// Win32s support:
//	tests for Win32 or Win32s,
// 	under Win32s uses a file instead of the registry database
//	


class Registry {

private:
	HKEY	theKey;
	FILE 	*theFile;
	char	theFileName[128];
	BOOL	Win32s;

	void DeleteKey(const char *key);
 	int QueryValueEx(const char *key,DWORD *Type,unsigned char FAR *data,unsigned long *size);
	int SetValueEx(const char *key,DWORD Type, unsigned char FAR *data, unsigned long length);



public:

	Registry();
	virtual ~Registry();

	void Place(const char *key,const char FAR *data);			// Place a subkey string into this key
	void Place(const char *key, void FAR *data,int length);	// Place a subkey block of data into this key
	void Place(const char *key, WORD data);					// Place a WORD subkey

	BOOL Get(const char *key, char FAR *data, unsigned int length);	// Get a subkey block of string
	BOOL Get(const char *key, CString &data);							// Get a subkey string
	BOOL Get(const char *key, void *data, unsigned int length);		// Get a subkey block of data
	BOOL hGet(const char *key,HANDLE hdata,unsigned int length);		// Get data into a global mem handle
	BOOL Get(const char *key, WORD *data);							// Get a WORD value


	// These are some standard functions which use the registry
	void SavePrinterMode(HANDLE theDevMode,HANDLE thePrinterName);
	void SavePageSize(CPoint tbePageSize);

	static CString getKey();

};


// Associate this software with some documents
void Associate(const char *extension,const char *title,const char *program);


////// The Find File class ////

class FindFile {
		HANDLE	handle;
		WIN32_FIND_DATA fileinfo;

public:
	FindFile(CString);
	virtual ~FindFile();
	BOOL 	FindNext();
	char 	*GetName();
	BOOL	Success();
};



#endif

