/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"

#include "TinyCad.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "TinyCadDoc.h"
#include "TinyCadView.h"
#include "Registry.h"
#include "LibraryView.h"
#include "LibraryDoc.h"
#include "TinyCadSymbolDoc.h"


#include "Object.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp

BEGIN_MESSAGE_MAP(CTinyCadApp, CWinApp)
	//{{AFX_MSG_MAP(CTinyCadApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(IDM_LIBLIB, OnLiblib)
	ON_COMMAND(ID_HELP_GOTOTINYCADWEBSITE, OnHelpGototinycadwebsite)
	ON_COMMAND(ID_HELP_HELP, OnHelpHelp)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp construction

CTinyCadApp::CTinyCadApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
	m_libhead = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTinyCadApp object

CTinyCadApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp initialization

BOOL CTinyCadApp::InitInstance()
{
	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("TinyCAD"));

	LoadStdProfileSettings(6);  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	m_pDocTemplate = new CMultiDocTemplate(
		IDR_TCADTYPE,
		RUNTIME_CLASS(CTinyCadDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTinyCadView));
	AddDocTemplate(m_pDocTemplate);

	m_pLibTemplate = new CMultiDocTemplate(
		IDR_LIBTYPE,
		RUNTIME_CLASS(CLibraryDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CLibraryView));
	AddDocTemplate(m_pLibTemplate);


	// Read the registry
	RegistryRead();


	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);


	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	if (m_Maximize && m_nCmdShow == 1)
	{
		m_nCmdShow = SW_SHOWMAXIMIZED;
	}

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CTinyCadApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp message handlers


// This is the idle time processing....
BOOL CTinyCadApp::OnIdle(LONG count)
{
  CWinApp::OnIdle(count);

  switch (count) {
	case 0:
		// Close all the method files
		{
		CLibraryFile *pointer = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();
		while (pointer!=NULL) {
			pointer->CloseMethodFile();
			pointer=pointer->next;
		}
		return TRUE;
		}
	default: // Sort the objects in the design into order

		// Do not sort if some objects are selected!
		/*
		if (thisDesign->IsSelected())
			return FALSE;

		return thisDesign->Sort();
		*/
		;
  }

  return FALSE;
}


int CTinyCadApp::ExitInstance() 
{
	// Delete the libraries
	CLibraryFile *pointer = GetLibHead();
	while (pointer != NULL) 
	{
		CLibraryFile *next = pointer->next;
		delete pointer;
		pointer = next;
	}
	SetLibHead(NULL);
	
	return CWinApp::ExitInstance();
}

// Read the registry
void CTinyCadApp::RegistryRead()
{
  Registry theRegistry;
  CString GetString;


  // Does the registry information exist - if not create it
  if (!theRegistry.Get("PageSize",GetString))
	RegistryInit();

  int rc = TRUE;



  // Is there a list of libraries in the registry?
  if (theRegistry.Get("Libraries",GetString)) {
	CString LibName;
	CLibraryFile *NewLib;
	int Fd;
	while (GetString.GetLength()!=0) {
		Fd = GetString.Find(',');
		if (Fd==-1) {
			LibName = GetString;
			GetString = "";
		} else {
			LibName = GetString.Left(Fd);
			GetString = GetString.Mid(Fd+1);
		}
		NewLib = new CLibraryFile;
		NewLib->Attach(LibName);
		NewLib->next = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();
		SetLibHead(NewLib);
	}
  }

  WORD hold;
  if (theRegistry.Get("Maximize", &hold))
  {
	  m_Maximize = hold;
  }
  else
  {
	  m_Maximize = FALSE;
  }

  if (theRegistry.Get("MdiMaximize", &hold))
  {
	  m_MdiMaximize = hold;
  }
  else
  {
	  m_MdiMaximize = FALSE;
  }


  if (theRegistry.Get("ShowPosition", &hold))
  {
	  m_ShowPosition = hold;
  }
  else
  {
	  m_ShowPosition = FALSE;
  }

  if (theRegistry.Get("PrintBandW", &hold))
  {
	  m_PrintBandW = hold;
  }
  else
  {
	m_PrintBandW = FALSE;
  }

  if (theRegistry.Get("PrintScale", &hold))
  {
	  m_PrintScale = hold;
  }
  else
  {
	m_PrintScale = 100;
  }

}



// Add default information to the registry
void CTinyCadApp::RegistryInit()
{

  // Display the welcome dialog
  CDialog theDialog(IDD_WELCOME);
  theDialog.DoModal();

  Registry theRegistry;

  // Save the page size setup
  theRegistry.SavePageSize(CSize(  297 * PIXELSPERMM, 210 * PIXELSPERMM ));

  theRegistry.Place( "Maximize", (WORD)FALSE );

  	TCHAR theBuffer[1024];
	DWORD theBytes = GetModuleFileName(NULL,theBuffer,sizeof(theBuffer) - 1);

	if (theBytes == 0)
		return;

	TCHAR* thePtr = theBuffer + theBytes;

	while (thePtr >= theBuffer && *thePtr != _TCHAR('\\'))
	{
		thePtr--;
	}
	*thePtr = _TCHAR('\0');
	CString theMainDir = theBuffer;
	theMainDir += "\\";


  // Search the default library directory for libraries
  FindFile theFind( theMainDir+"library\\*.idx" );

  if (theFind.Success()) {
	  CString Libraries = "";

	do {
		// Add this library to the list of libraries in use
		if (Libraries != "")
			Libraries += ",";
		Libraries += theMainDir+"library\\"+theFind.GetName();;
		// Remove the extension (of .idx)
		Libraries = Libraries.Left(Libraries.ReverseFind('.'));
	} while (theFind.FindNext());

	theRegistry.Place("Libraries",Libraries);
  }

  // Finially place the association of this file and the design type
  Associate(".dsn","TinyCAD design",AfxGetAppName());
}


void CTinyCadApp::SetMaximize( BOOL m )
{
	Registry theRegistry;
	theRegistry.Place( "Maximize", m );
	m_Maximize = m;
}

void CTinyCadApp::SetMdiMaximize( BOOL m )
{
	Registry theRegistry;
	theRegistry.Place( "MdiMaximize", m );
	m_MdiMaximize = m;
}

void CTinyCadApp::SetPrintBandW( BOOL m )
{
	Registry theRegistry;
	theRegistry.Place( "PrintBandW", m );
	m_PrintBandW = m;
}

void CTinyCadApp::SetPrintScale( int m )
{
	Registry theRegistry;
	theRegistry.Place( "PrintScale", m );
	m_PrintScale = m;
}


// Edit a library using the doc/view
void CTinyCadApp::editLibrary(CLibraryFile *library)
{
	// Is this library already being edited?
	POSITION p;
	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		if (t->IsLibInUse( library ))
		{
			POSITION v = t->GetFirstViewPosition();
			
			// Active it's views
		   while (v != NULL)
		   {
			  CView* pView = t->GetNextView(v);
			  pView->GetParentFrame()->BringWindowToTop();
		   }   

		   return;
		}		
	}

	
	CLibraryDoc *pDoc = static_cast<CLibraryDoc *>(m_pLibTemplate->CreateNewDocument());

	if (pDoc != NULL)
	{
		pDoc->setLibrary( library );
		CFrameWnd *pFrame = m_pLibTemplate->CreateNewFrame( pDoc, NULL );
		m_pLibTemplate->InitialUpdateFrame( pFrame, pDoc, TRUE );
	}
}

CLibraryFileSymbol *CTinyCadApp::Extract(const char *key)
{
  CLibraryFileSymbol *theSymbol = NULL;
  CLibraryFile *LibPointer=GetLibHead();

  while (theSymbol==NULL && LibPointer!=NULL) {
	theSymbol = LibPointer->Extract(key);
	LibPointer=LibPointer->next;
  }

  return theSymbol;
}



////// the Library Menu //////

void CTinyCadApp::OnLiblib() 
{
  CDlgLibraryBox theDialog(AfxGetMainWnd());

  theDialog.DoModal();
}


void CTinyCadApp::ResetAllSymbols()
{
	POSITION p;

	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		t->ResetSymbols();
	}
}




bool CTinyCadApp::IsLibInUse( CLibraryFile *lib )
{

	POSITION p;
	p = m_pDocTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CTinyCadDoc *t = static_cast<CTinyCadDoc*>(m_pDocTemplate->GetNextDoc(p));
		if (t->IsLibInUse( lib ))
		{
			return true;
		}
	}

	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		if (t->IsLibInUse( lib ))
		{
			return true;
		}		
	}

	return false;
}

// Edit a library using the doc/view
void CTinyCadApp::editSymbol(CLibraryFile *library, CLibraryFileSymbol &symbol)
{
	CTinyCadSymbolDoc *pDoc = new CTinyCadSymbolDoc();
	pDoc->setSymbol( library, symbol );

	CFrameWnd *pFrame = m_pDocTemplate->CreateNewFrame( pDoc, NULL );
	m_pDocTemplate->InitialUpdateFrame( pFrame, pDoc, TRUE );
}

void CTinyCadApp::OnHelpGototinycadwebsite() 
{
	// Open a browser for our web site
	ShellExecute( AfxGetMainWnd()->m_hWnd, "open", "http://tinycad.sourceforge.net", NULL, NULL, SW_SHOWNORMAL );	
}



